module Elevatore
  class Elevator
    attr_reader :current_time, :current_floor, :people, :direction
    def initialize opts={}
      @current_floor = opts[:current_floor] || 0
      @people = opts[:people]
      @current_time = 0
      @direction = opts[:direction] || Elevatore::DIRECTION_UP
      if @direction != Elevatore::DIRECTION_UP && @direction != Elevatore::DIRECTION_DOWN
        raise ArgumentError.new("Your direction is misleading the elevator")
      end
    end

    def print_exit_times
      clear_building!
      output_people!
    end

    private
    def clear_building!
      while @people.any?{|person| has_inside?(person) || is_being_called?(person)}
        people_exit!
        people_enter!
        set_direction!
        travel!
      end
    end

    def travel!
      @current_time += Elevatore::SPEED
      @current_floor += @direction if needs_to_roll?
    end

    def people_exit!
      people_inside.each do |person|
        person.exit_at = @current_time.round(1) if @current_floor == person.to
      end
    end

    def people_enter!
      people_to_enter.each do |person|
        if @current_floor == person.from && person.pickup_at.nil? && person.enter_at == @current_time.round(1)
          person.pickup_at = @current_time.round(1)
        end
      end
    end

    def output_people!
      people.sort_by(&:exit_at).each do |person|
        puts "#{person.exit_at}m P#{person.id}"
      end
    end

    def needs_to_roll?
      ((Elevatore::TOP_LEVEL >= @current_floor+1 && @direction == Elevatore::DIRECTION_UP) || (Elevatore::LOWER_LEVEL <= @current_floor-1 && @direction == Elevatore::DIRECTION_DOWN)) && ((!next_pickup.nil? && people_inside.size != 0) || next_pickup != @current_floor)
    end

    def is_being_called? person
      !has_picked_up?(person) && !has_delivered?(person)
    end

    def has_inside? person
      has_picked_up?(person) && !has_delivered?(person)
    end

    def has_picked_up? person
      !person.pickup_at.nil?
    end

    def has_delivered? person
      has_picked_up?(person) && !person.exit_at.nil?
    end

    def people_inside
      @people.select{|person| has_inside?(person) }
    end

    def people_to_enter
      @people.select{|person| is_being_called?(person) }
    end

    def next_pickup
      people_to_enter.min_by{|person| person.enter_at }&.from
    end

    def next_drop
      if @direction==Elevatore::DIRECTION_UP
        people_inside.max_by{|person| person.to }&.to
      else
        people_inside.min_by{|person| person.to }&.to
      end
    end

    def set_direction!
      return if (next_pickup.nil? && people_inside.size == 0) || (next_drop.nil? && people_to_enter.size == 0)
      if @direction==Elevatore::DIRECTION_UP
        unless (next_pickup.to_i > @current_floor && people_inside.size == 0) || (next_drop.to_i > @current_floor)
          @direction=Elevatore::DIRECTION_DOWN
        end
      else
        unless (next_pickup.to_i < @current_floor && people_inside.size == 0) || (next_drop.to_i < @current_floor)
          @direction=Elevatore::DIRECTION_UP
        end
      end
    end
  end
end
